__title__ = 'byteplus-rec-core'
__description__ = 'core pkg for byteplus rec'
__url__ = 'https://github.com/byteplus-sdk'
__version__ = '1.0.3'
__author__ = 'Guowei Lei'
__author_email__ = 'leiguowei@bytedance.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2021 Bytedance'
